
# q-expansion of newform 667.2.a.c, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 667
weight = 2
poly_data = [-40, -88, 219, 352, -547, -402, 641, 97, -298, 24, 58, -11, -4, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-3, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[4, 4, -6, -1, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-363, 153, 1894, -1566, -2433, 2882, 604, -1530, 147, 313, -67, -22, 6], 19], [[-28, -613, -236, 3232, 233, -4758, 818, 2387, -621, -482, 145, 34, -11], 19], [[-104, -765, -198, 3365, 233, -4777, 818, 2387, -621, -482, 145, 34, -11], 19], [[-47, -860, -217, 3384, 233, -4777, 818, 2387, -621, -482, 145, 34, -11], 19], [[3, -942, -865, 3869, 1379, -5121, 44, 2447, -382, -479, 110, 33, -9], 19], [[516, 1338, -2024, -4377, 3203, 4512, -2464, -1923, 910, 357, -156, -24, 10], 19], [[-1326, -2985, 4344, 10979, -6078, -13307, 5613, 6176, -2422, -1196, 457, 82, -31], 38], [[-449, -1628, 1313, 6887, -2400, -8947, 3092, 4272, -1512, -839, 301, 58, -21], 19], [[-1136, -3175, 3736, 11397, -5774, -13478, 5575, 6195, -2422, -1196, 457, 82, -31], 19]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1], [1, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0], [1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0], [2, -1, 0, 0, -1, 0, 1, 0, 0, 0, 1, -1, 0], [0, -1, 0, 0, 0, 0, 2, -1, 0, 0, -1, -1, 0], [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [-1, 2, 0, 1, 0, 0, -2, 2, 0, 0, 0, 1, 0], [1, 0, 0, -1, 0, 1, 0, 0, -1, -1, -1, 0, 2], [1, 0, -1, 0, 1, 0, 0, 0, -1, 1, 0, 0, 2], [0, 0, 0, 0, -1, -1, -1, 1, -1, 0, 1, 1, 1], [2, -3, 1, 0, 0, 0, 0, 0, 0, -1, 0, -1, 0], [4, -3, 1, 0, -1, 0, 2, -1, 1, 0, 0, -2, 0], [0, -1, 1, -1, 0, 4, 0, 1, 1, -2, 2, 0, -1], [2, -1, 0, 2, -1, -1, -1, 0, 0, 0, 2, -1, -2], [1, -1, -1, 1, 1, 0, -1, 1, -1, -2, 2, 1, -1], [2, -3, 0, 1, -1, -3, 0, -1, 1, -1, -2, 0, -1], [2, -1, -1, 1, 0, -1, 2, 0, -2, 1, 0, 1, 0], [0, 2, 0, -4, 2, 0, -1, 0, 2, 0, 1, 0, 1], [1, 2, 0, -1, 2, -2, -3, 0, 2, 1, 0, 1, 2], [4, 1, 2, -1, 1, 0, -1, 0, 3, 0, -3, 0, 2], [4, 0, 1, 1, 1, 0, 0, -2, 1, 0, 0, 1, 0], [3, -2, 2, -1, -1, 3, 3, 1, 0, -2, 4, -1, -1], [1, 2, -1, 1, 0, 0, 2, -1, 2, -2, 0, 1, -1], [-2, 3, -2, 0, 0, 0, 1, 1, 1, 2, -2, 0, -1], [-1, 2, 1, 3, -1, -3, -1, 0, -1, 1, -2, 0, -1], [4, -1, 0, 1, 1, 0, -2, -2, 1, -2, -2, 0, -2], [2, -2, 1, 0, -1, 2, 2, -2, 1, 1, -2, -1, 1], [0, 1, 0, 4, 2, -3, 0, 1, 2, 4, -2, -1, -2], [-1, 2, 0, 2, 2, 1, 0, 2, 0, 1, -1, -1, -1], [-3, 0, 0, -3, -1, 2, 0, -1, 1, 0, 0, 0, 3], [5, 1, 2, 3, 2, 0, -2, 1, 1, -1, 0, 3, 0], [-2, -4, -1, -1, -3, -2, -1, -1, 0, -1, 0, 0, -1], [4, -2, 2, 0, 0, 1, -1, 0, -1, 0, -1, 1, 1], [-4, -1, -3, 3, 1, -3, 0, -1, -1, 0, -4, 0, 1], [-2, 3, 0, 2, 2, -5, 0, 0, -1, 2, -4, 0, 0], [0, 3, -1, 0, 0, -1, -1, -1, 3, 0, 2, 0, 0], [2, -2, 3, 0, -1, 1, 0, -1, 1, -1, 2, -3, -1], [0, -3, -1, -1, -2, 0, 3, -2, -1, 0, -4, -4, 2], [-3, -1, -1, -1, -1, -2, -5, 1, -1, -1, -1, 1, -2], [1, 2, 2, 0, 1, -2, -1, 1, 0, -1, 4, 1, 1], [6, 0, -4, -1, -2, 4, 2, 1, 0, 0, 4, 2, 0], [-5, 7, 0, -2, 0, 1, -3, 1, -2, 0, -2, 4, 1], [-2, 1, 1, 1, 2, 0, -1, -2, 0, 1, 0, -1, 1], [6, 2, 0, -4, 2, 1, -4, -1, 1, -4, 2, 1, -2], [2, 2, 1, -5, -1, 0, -5, 2, -1, 0, 4, 1, 5], [2, 6, -1, 3, 1, -1, -2, 0, 5, 4, 1, 1, -3], [2, 0, 1, 0, -1, -2, -2, 1, 1, 1, 4, -3, 2], [-2, 0, 0, 3, 2, -1, 1, -1, -2, 3, -6, 2, -1], [2, 1, 1, 6, 0, -4, -2, 2, 1, 3, -4, 0, 1], [3, 0, -2, -1, -3, 3, 2, -1, 0, 1, 5, -1, 2], [11, -1, 1, -1, 1, -1, -4, 2, 4, -3, 2, 2, 0], [3, -3, -1, 2, -1, 1, 2, -3, 1, -2, -4, -1, 1], [8, 5, -1, 2, 1, 0, -3, 1, 2, 0, 0, 5, 3], [-7, 10, -5, 1, 2, 1, -1, 3, -1, 1, 2, 4, 2], [4, 0, 1, 2, 0, 2, 2, 2, 1, -1, 4, -1, 0], [0, 7, -4, 4, 3, -2, -1, 2, 0, 4, -4, 1, -4], [-1, -1, 2, 1, -2, 1, 5, 1, -2, -1, -2, 0, -1], [12, -2, -4, -1, -2, -2, 1, -2, 0, 0, -1, 0, 0], [-4, -6, -1, -5, -1, 2, -1, -2, 3, -3, 2, -1, 2], [0, -2, -6, -2, -4, -2, 0, -2, -7, -1, -2, 1, -2], [-4, 4, -2, 4, -3, 1, -2, 1, -3, 1, -4, 1, -1], [6, -1, 1, 5, 0, -4, 0, 0, -1, 6, -8, 0, 2], [-2, 2, 2, 5, 2, -4, -4, 2, 0, 3, -2, -4, -3], [2, -1, -1, 3, 1, 2, -1, -1, -2, 2, -5, 1, 0], [-8, 0, -1, 4, -1, -3, 2, 1, 0, 2, -2, 2, -2], [-1, -3, 7, 1, -2, 2, 4, -1, 0, 0, -2, -4, -2], [4, 0, 3, 4, 3, 3, -2, 2, 2, -1, -2, 4, -3], [-4, -1, 0, -2, 1, 3, 1, -2, 2, 0, 5, -3, 2], [6, -1, 0, -5, -1, -1, -2, -2, -2, -4, 0, 3, 1], [2, 3, -4, -7, 1, -1, -2, 2, 3, -3, 4, 0, 2], [5, 1, -1, -2, 5, -1, -4, 3, 4, -2, 5, 2, 0], [5, 6, 0, 1, 1, 4, 1, 1, 2, -3, 8, 1, -3], [-1, 3, -3, -3, 2, -2, -1, 2, 0, 2, -1, 0, -1], [3, 1, -3, -4, -2, 0, 0, -4, 4, 1, 6, 2, -4], [-2, 1, -4, 3, 0, 0, 4, -1, -1, 0, 2, 2, 0], [-3, 1, 4, -3, -2, 3, 1, 3, -4, -2, -4, 0, 1], [2, 0, 0, 1, -2, 2, -3, 0, -3, -1, -2, 1, 3], [8, 0, -2, -2, -1, 6, -2, 3, 3, -5, 7, -1, -6], [-2, 1, -5, -4, 0, -1, 4, -4, 1, 0, 6, -2, -4], [1, 1, 4, -1, 1, -4, -8, 2, 0, 1, -6, 0, 0], [-7, 0, -2, 1, 1, -4, 3, 0, 4, 5, 0, -1, -6], [3, 8, 1, -8, -2, 3, -3, 3, -3, -3, 0, 2, 7], [-5, 4, 0, 4, 0, -3, 1, -4, 2, -1, 0, -1, -6], [-4, 6, -2, -4, 1, 3, -7, 1, -3, -4, 4, 3, -1], [1, 3, 0, -1, 0, 1, -1, 1, -1, -3, 6, 3, -5], [-8, 0, 4, 0, -1, 6, -2, 2, -4, 0, -2, 2, 1], [4, -2, 1, 3, -1, 7, 5, 0, 1, 0, 3, -3, -1], [2, -4, -1, 0, -4, -6, 1, 1, -4, 1, 0, -2, 1], [-6, -5, -1, 0, -5, 5, 8, 1, -4, -2, -2, -3, 2], [-7, 3, -1, -8, 1, 2, 3, -3, 1, 1, 0, -1, 5], [-2, -5, -3, 3, 0, -4, 3, -5, 0, -1, -2, -1, 0], [2, 7, 4, -5, -2, 5, 2, -1, 1, -3, 1, 0, 4], [7, 0, -5, -2, 1, -1, -1, -1, -2, 2, -2, 1, 6], [2, 6, 2, -6, 0, 5, 2, -2, -1, -1, 4, -1, 2], [-6, 3, -3, -2, 2, -2, -7, 0, -1, -4, 2, 6, 0], [-1, 6, 0, -2, 7, -1, -2, 3, 3, 3, 0, 0, 5], [-2, 3, 0, 6, 0, -3, -3, -2, -2, 1, -8, -3, 1], [-1, 5, 4, 5, 7, -7, -6, 4, 2, 5, -8, 0, -2], [9, 0, -4, 1, 1, -2, -5, -1, 1, 4, -7, 0, 6], [2, -10, 3, 1, -2, 0, 4, 3, 4, 0, 2, -2, -4], [-2, -11, 0, -2, -5, 1, 9, -2, -6, 1, -2, -7, 0], [-2, 1, -4, -4, 3, -3, -3, 0, 3, 0, 2, -2, -5], [2, 1, 1, 4, 3, -3, -1, 0, -6, 2, -4, 3, 8], [5, -1, 1, -2, -2, 5, 6, 3, 3, -1, 4, 1, 1], [0, 2, -5, -1, 6, 0, 1, -5, 4, 1, 2, -2, 0], [-5, -8, -7, 2, -7, -4, 5, -4, -5, 3, 0, -2, 0], [-16, 0, 0, -2, -4, 1, 3, 0, -1, 1, 0, 1, 0], [-1, -3, -3, -3, -1, 5, 2, -1, -7, -3, 6, 1, -5], [-2, 4, 4, -6, 3, 4, -5, 3, 5, -1, 8, -1, -3], [-4, -5, 6, 0, -2, 5, 2, 3, 2, -2, -1, -3, 5], [-2, -4, 3, 2, 0, 2, 2, -1, 3, -3, 2, -1, -2], [9, -5, 0, -3, -2, 5, 0, 3, 0, -7, 3, 2, -3], [-4, -10, 1, -5, -3, -5, 0, -4, -4, -1, -6, -2, 2], [-6, -3, 5, -2, -1, 0, 5, 2, -7, 0, 0, -4, 4], [5, 4, -6, 4, -2, -2, -1, -2, -4, 0, -8, 3, 2], [-5, 7, 3, -2, 4, -4, -10, 4, -2, 1, -1, 4, 0], [-2, 2, -1, 3, 4, -6, 1, 3, -2, 4, -10, 0, 4], [0, -8, 5, 0, -4, 8, 5, 0, -1, -4, 8, -3, -1], [7, -7, 2, -4, -5, 2, 0, -4, -1, 0, 0, -1, 3], [0, 2, 6, 0, -2, 4, 5, 6, 1, -3, 4, -3, 1], [-19, 2, -2, -3, 6, -9, -11, 4, 1, 3, 0, 6, -2], [8, -1, 4, 6, 4, -4, -3, 3, 4, 5, 0, -3, 0], [3, -7, -3, -5, 2, 8, 1, 0, -1, -6, 6, 3, -5], [7, -2, 4, 3, 3, -5, 1, -4, -1, 0, -12, -4, 7], [3, -4, -2, -6, -1, 2, -2, 3, 0, -2, 8, -1, -1], [-3, 3, -3, 1, -2, 1, -1, 4, -2, -3, -2, 0, -3], [-4, -9, 7, -9, 0, 3, -6, 0, -4, -8, 0, -1, 7], [-8, -1, 4, -6, 3, 5, 1, -2, 7, -5, 1, 0, 2], [5, -7, 4, 1, 1, 7, 4, -4, 0, 1, 3, -2, 4], [2, -9, 1, -1, -1, 3, 9, -1, 2, 4, -1, -3, 4], [-3, -1, 9, 0, 1, 1, -4, 2, 1, -4, 2, -1, -3], [2, -3, 4, 1, 0, -1, -5, 3, 0, 1, 11, -3, -1], [6, 2, -4, 2, -2, -5, 1, -4, -5, -2, -6, -1, -2], [-6, -2, -2, 4, -4, 1, 5, -2, 0, -6, 0, 0, -6], [10, 8, 0, 0, 2, 0, 3, 0, -2, -1, -8, -2, 4], [7, 0, -7, -1, 6, -5, -6, -5, 5, -5, 2, 8, -3], [-8, 8, -3, 2, 3, -4, 3, -2, 6, 1, -8, 2, 6], [1, 5, 5, 2, -2, -4, -3, -1, 10, 0, -2, 0, 0], [-1, -5, -6, -6, -6, 7, 1, -1, -5, -1, 18, 3, -1], [2, 9, -1, 3, 1, 3, -2, 3, 1, 0, -1, 2, 14], [7, -16, 0, -2, -4, 5, 11, -6, 9, -6, 6, -8, -6], [-2, -4, -11, 1, -1, 0, 5, -4, -8, -2, 0, -2, -2], [7, 2, -5, 5, 1, -8, -3, 2, -2, 5, -8, 5, -2], [5, 2, 0, -1, 4, -1, -12, 8, 5, 2, 4, 4, 3], [1, 1, 8, 4, 0, 3, 3, 1, 6, 0, -4, -4, 7], [6, 11, -8, 0, -1, 1, 4, 0, 0, -2, 8, -3, -3], [-6, -2, -2, 4, 0, -1, 6, -7, -5, 6, -6, 1, 3], [-9, 1, -7, -7, 4, -6, -1, -4, 1, -1, -9, 1, 4], [0, -7, -2, 2, -4, -7, -4, 0, -3, 8, -8, -2, 3], [-9, -2, 4, 0, 2, -4, -3, 2, 1, 9, -4, 0, 6], [-2, -4, -10, -2, -8, 4, 6, -2, 0, 0, 5, 2, 5], [6, 0, 3, 1, 4, 0, 6, -1, -1, 6, -1, -1, 1], [5, 1, -1, -2, 8, 1, 1, 0, 6, 0, 4, 0, 2], [-10, -4, -1, -4, 0, 2, 3, -4, -4, -5, 4, 2, -4], [4, -9, 3, -4, 1, 2, 1, -3, 2, -8, -4, -1, 12], [6, 2, 2, 13, 3, -8, -6, 7, 2, 5, -4, 4, -3], [-6, 2, 2, -3, 1, -3, -11, 5, 2, 1, 10, -2, -2], [-10, 7, 1, 2, -3, 2, 2, 3, 1, 1, 0, -6, -5], [-4, -5, 2, 4, -3, 6, 5, 2, 5, 0, 4, -6, -7], [-8, 12, -3, -4, 3, -1, -4, 2, 4, 1, 6, 4, -1], [3, 4, 1, 3, 5, -6, -2, 2, 7, 6, -14, -2, 5], [3, -12, -1, 6, 2, -1, -1, -1, 0, 2, 1, -1, -4], [0, 9, -13, 1, 2, -1, 1, -4, -2, 2, 2, -1, -2], [-4, 10, -2, -6, 2, -1, -6, 4, 3, 1, 8, 1, -5]]
